/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Predicate;

import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.LinkRelation;
import org.springframework.hateoas.server.RepresentationModelProcessor;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.attachment.AttachmentByVorgangController;
import de.ozgcloud.alfa.common.command.CommandController.CommandByRelationController;
import de.ozgcloud.alfa.common.user.UserManagerUrlProvider;
import de.ozgcloud.alfa.historie.HistorieController;
import de.ozgcloud.alfa.kommentar.KommentarController.KommentarByVorgangController;
import de.ozgcloud.alfa.postfach.PostfachMailController;
import de.ozgcloud.alfa.representation.RepresentationByVorgangController;
import de.ozgcloud.alfa.vorgang.VorgangProperties.VorgangProperty;
import lombok.RequiredArgsConstructor;

@Component
@RequiredArgsConstructor
class VorgangWithEingangProcessor implements RepresentationModelProcessor<EntityModel<VorgangWithEingang>> {

	static final LinkRelation REL_KOMMENTARE = LinkRelation.of("kommentare");
	static final LinkRelation REL_ATTACHMENTS = LinkRelation.of("attachments");
	static final LinkRelation REL_DOWNLOAD_ATTACHMENTS = LinkRelation.of("downloadAttachments");
	static final LinkRelation REL_REPRESENTATIONS = LinkRelation.of("representations");
	static final LinkRelation REL_POSTFACH_MAILS = LinkRelation.of("postfachMails");
	static final LinkRelation REL_VORGANG_FORWARDING = LinkRelation.of("forwarding");
	static final LinkRelation REL_HISTORIE = LinkRelation.of("historie");
	static final LinkRelation REL_PROCESS_VORGANG = LinkRelation.of("processVorgang");

	static final String REL_SEARCH_USER = "search-user-profiles";
	static final String USER_PROFILE_SEARCH_DELETED_PARAM = "deleted";
	static final String USER_PROFILE_SEARCH_ORGANISATIONS_EINHEIT_ID_PARAM = "organisationsEinheitId";

	private final PostfachMailController postfachMailController;
	private final UserManagerUrlProvider userManagerUrlProvider;

	private final VorgangProcessorProperties vorgangProcessorProperties;

	private static final Predicate<VorgangWithEingang> HAS_ATTACHMENTS = vorgangWithEingang -> vorgangWithEingang.getEingang()
			.getNumberOfAttachments() > 0;

	private static final Predicate<VorgangWithEingang> HAS_REPRESENTATIONS = vorgangWithEingang -> vorgangWithEingang.getEingang()
			.getNumberOfRepresentations() > 0;

	@Override
	public EntityModel<VorgangWithEingang> process(EntityModel<VorgangWithEingang> model) {
		var vorgang = model.getContent();

		if (Objects.isNull(vorgang)) {
			return model;
		}

		return model
				.add(linkTo(KommentarByVorgangController.class).slash(vorgang.getId()).slash("kommentars").withRel(REL_KOMMENTARE))
				.addAllIf(HAS_ATTACHMENTS.test(vorgang), () -> List.of(
						linkTo(methodOn(AttachmentByVorgangController.class).getAllByVorgang(vorgang.getId())).withRel(REL_ATTACHMENTS),
						linkTo(methodOn(AttachmentByVorgangController.class).download(vorgang.getId())).withRel(REL_DOWNLOAD_ATTACHMENTS)))
				.addIf(HAS_REPRESENTATIONS.test(vorgang),
						() -> linkTo(methodOn(RepresentationByVorgangController.class).getAll(vorgang.getId())).withRel(REL_REPRESENTATIONS))
				.addIf(isPostfachConfigured(),
						() -> linkTo(methodOn(PostfachMailController.class).getAll(vorgang.getId())).withRel(REL_POSTFACH_MAILS))
				.add(linkTo(methodOn(HistorieController.class).getAll(vorgang.getId())).withRel(REL_HISTORIE))
				.addIf(userManagerUrlProvider.isConfiguredForSearchUserProfile() && hasOrganisationsEinheitId(vorgang),
						() -> buildSearchUserProfilesLink(vorgang))
				.addIf(isProcessable(vorgang),
						() -> linkTo(methodOn(CommandByRelationController.class).createCommand(vorgang.getId(), vorgang.getId(), vorgang.getVersion(),
								null)).withRel(REL_PROCESS_VORGANG));
	}

	private boolean isPostfachConfigured() {
		return postfachMailController.isPostfachConfigured();
	}

	private Link buildSearchUserProfilesLink(VorgangWithEingang vorgang) {
		return Link.of(userManagerUrlProvider.getSearchUserProfilesTemplate()
				.queryParam(USER_PROFILE_SEARCH_DELETED_PARAM, false)
				.queryParam(USER_PROFILE_SEARCH_ORGANISATIONS_EINHEIT_ID_PARAM, getOrganisationsEinheitId(vorgang))
				.build(false).toUriString(), REL_SEARCH_USER);
	}

	private boolean hasOrganisationsEinheitId(VorgangWithEingang vorgang) {
		return Optional.of(vorgang.getEingang()).map(Eingang::getZustaendigeStelle).map(ZustaendigeStelle::getOrganisationseinheitenId).isPresent();
	}

	private String getOrganisationsEinheitId(VorgangWithEingang vorgang) {
		return vorgang.getEingang().getZustaendigeStelle().getOrganisationseinheitenId();
	}

	private boolean isProcessable(VorgangWithEingang vorgang) {
		return isAnyFormIdAndFormEngineNameMatching(vorgang.getEingang().getHeader(), vorgangProcessorProperties.getProcessor());
	}

	private boolean isAnyFormIdAndFormEngineNameMatching(EingangHeader eingangHeader, List<VorgangProperty> properties) {
		return properties.stream().anyMatch(prop -> isFormIdAndFormEngineNameMatching(eingangHeader, prop));
	}

	private boolean isFormIdAndFormEngineNameMatching(EingangHeader eingangHeader, VorgangProperty property) {
		return property.getFormId().equals(eingangHeader.getFormId()) && property.getFormEngineName().equals(eingangHeader.getFormEngineName());
	}
}