/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.List;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import com.thedeanda.lorem.LoremIpsum;

import lombok.SneakyThrows;

class OrganisationsEinheitControllerTest {

	@InjectMocks
	private OrganisationsEinheitController controller;

	@Mock
	private OrganisationsEinheitService service;

	@Mock
	private OrganisationsEinheitModelAssembler assembler;

	@Mock
	private OrganisationsEinheitHeaderModelAssembler headerModelAssembler;

	private MockMvc mockMvc;

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestGetById {

		private final OrganisationsEinheit organisationsEinheit = OrganisationsEinheitTestFactory.create();

		@BeforeEach
		void mockService() {
			when(service.getById(OrganisationsEinheitTestFactory.ID)).thenReturn(organisationsEinheit);
		}

		@SneakyThrows
		@Test
		void shouldCallOrganisationsEinheitService() {
			performRequest();

			verify(service).getById(OrganisationsEinheitTestFactory.ID);
		}

		@Test
		void shouldCallAssembler() {
			performRequest();

			verify(assembler).toModel(organisationsEinheit);
		}

		@SneakyThrows
		@Test
		void shouldReturnStatusOk() {
			when(assembler.toModel(organisationsEinheit)).thenReturn(EntityModel.of(organisationsEinheit));

			var response = performRequest();

			response.andExpect(status().isOk());
		}

		@SneakyThrows
		@Test
		void shouldHaveOrganisationsEinheitId() {
			when(assembler.toModel(organisationsEinheit)).thenReturn(EntityModel.of(organisationsEinheit));

			var response = performRequest();

			response.andExpect(jsonPath("$.id").value(OrganisationsEinheitTestFactory.ID));
		}

		@SneakyThrows
		@Test
		void shouldHaveOrganisationsEinheitXzufiId() {
			when(assembler.toModel(organisationsEinheit)).thenReturn(EntityModel.of(organisationsEinheit));

			var response = performRequest();

			response.andExpect(jsonPath("$.xzufiId.id").value(XzufiIdTestFactory.ID));
		}

		@SneakyThrows
		@Test
		void shouldHaveOrganisationsEinheitXzufiSchemeAgencyId() {
			when(assembler.toModel(organisationsEinheit)).thenReturn(EntityModel.of(organisationsEinheit));

			var response = performRequest();

			response.andExpect(jsonPath("$.xzufiId.schemeAgencyId").value(XzufiIdTestFactory.SCHEME_AGENCY_ID));
		}

		@SneakyThrows
		private ResultActions performRequest() {
			return mockMvc.perform(get(OrganisationsEinheitController.PATH + "/" + OrganisationsEinheitTestFactory.ID));
		}
	}

	@Nested
	class TestSearch {

		private final OrganisationsEinheitHeader organisationsEinheitHeader1 = OrganisationsEinheitHeaderTestFactory.create();
		private final OrganisationsEinheitHeader organisationsEinheitHeader2 = OrganisationsEinheitHeaderTestFactory.createBuilder()
				.name(LoremIpsum.getInstance().getName())
				.build();
		private final String searchBy = LoremIpsum.getInstance().getWords(5);
		private final List<OrganisationsEinheitHeader> organisationsEinheitHeaders = List.of(organisationsEinheitHeader1,
				organisationsEinheitHeader2);

		@BeforeEach
		void setUpMocks() {
			when(service.searchOrganisationsEinheiten(searchBy)).thenReturn(Stream.of(organisationsEinheitHeader1, organisationsEinheitHeader2));
			when(headerModelAssembler.toCollectionModel(organisationsEinheitHeaders))
					.thenReturn(
							CollectionModel.of(List.of(EntityModel.of(organisationsEinheitHeader1), EntityModel.of(organisationsEinheitHeader2))));
		}

		@Test
		void shouldCallService() {
			performRequest();

			verify(service).searchOrganisationsEinheiten(searchBy);
		}

		@Test
		void shouldCallModelAssembler() {
			performRequest();

			verify(headerModelAssembler).toCollectionModel(organisationsEinheitHeaders);
		}

		@SneakyThrows
		@Test
		void shouldReturnStatusOk() {
			var response = performRequest();

			response.andExpect(status().isOk());
		}

		@Test
		@SneakyThrows
		void shouldHaveFirstOrganisationsEinheitName() {
			var response = performRequest();
			response.andExpect(jsonPath("$.content[0].name").value(organisationsEinheitHeader1.getName()));
		}

		@Test
		@SneakyThrows
		void shouldHaveFirstOrganisationsEinheitStrasse() {
			var response = performRequest();
			response.andExpect(jsonPath("$.content[0].anschrift.strasse").value(organisationsEinheitHeader1.getAnschrift().getStrasse()));
		}

		@Test
		@SneakyThrows
		void shouldHaveFirstOrganisationsEinheitHausnummer() {
			var response = performRequest();
			response.andExpect(jsonPath("$.content[0].anschrift.hausnummer").value(organisationsEinheitHeader1.getAnschrift().getHausnummer()));
		}

		@Test
		@SneakyThrows
		void shouldHaveFirstOrganisationsEinheitPlz() {
			var response = performRequest();
			response.andExpect(jsonPath("$.content[0].anschrift.plz").value(organisationsEinheitHeader1.getAnschrift().getPlz()));
		}

		@Test
		@SneakyThrows
		void shouldHaveFirstOrganisationsEinheitOrt() {
			var response = performRequest();
			response.andExpect(jsonPath("$.content[0].anschrift.ort").value(organisationsEinheitHeader1.getAnschrift().getOrt()));
		}

		@Test
		@SneakyThrows
		void shouldHaveSecondOrganisationsEinheitName() {
			var response = performRequest();
			response.andExpect(jsonPath("$.content[1].name").value(organisationsEinheitHeader2.getName()));
		}

		@Test
		@SneakyThrows
		void shouldHaveSecondOrganisationsEinheitStrasse() {
			var response = performRequest();
			response.andExpect(jsonPath("$.content[1].anschrift.strasse").value(organisationsEinheitHeader2.getAnschrift().getStrasse()));
		}

		@Test
		@SneakyThrows
		void shouldHaveSecondOrganisationsEinheitHausnummer() {
			var response = performRequest();
			response.andExpect(jsonPath("$.content[1].anschrift.hausnummer").value(organisationsEinheitHeader2.getAnschrift().getHausnummer()));
		}

		@Test
		@SneakyThrows
		void shouldHaveSecondOrganisationsEinheitPlz() {
			var response = performRequest();
			response.andExpect(jsonPath("$.content[1].anschrift.plz").value(organisationsEinheitHeader2.getAnschrift().getPlz()));
		}

		@Test
		@SneakyThrows
		void shouldHaveSecondOrganisationsEinheitOrt() {
			var response = performRequest();
			response.andExpect(jsonPath("$.content[1].anschrift.ort").value(organisationsEinheitHeader2.getAnschrift().getOrt()));
		}

		@SneakyThrows
		private ResultActions performRequest() {
			return mockMvc.perform(get(OrganisationsEinheitController.PATH)
					.param("searchBy", searchBy)).andExpect(status().isOk());

		}
	}

	@Nested
	class TestGetOrganisationEinheit {

		@Test
		void shouldCallService() {
			controller.getOrganisationEinheit(OrganisationsEinheitTestFactory.ID);

			verify(service).getById(OrganisationsEinheitTestFactory.ID);
		}

		@Test
		void shouldReturnOrganisationEinheit() {
			var organisationEinheitFromService = OrganisationsEinheitTestFactory.create();
			when(service.getById(any())).thenReturn(organisationEinheitFromService);

			var organisationsEinheit = controller.getOrganisationEinheit(OrganisationsEinheitTestFactory.ID);

			assertThat(organisationsEinheit).isSameAs(organisationEinheitFromService);
		}
	}
}
