/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.Mock;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializerProvider;

import de.ozgcloud.alfa.common.user.UserId;
import de.ozgcloud.alfa.common.user.UserManagerUrlProvider;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import lombok.SneakyThrows;

class LinkedUserProfileResourceSerializerTest {

	private static final LinkedUserProfileResource ANNOTATION = getAnnotation();

	@Mock
	private UserManagerUrlProvider userManagerUrlProvider;

	private LinkedUserProfileResourceSerializer serializer;

	@BeforeEach
	void init() {
		serializer = LinkedUserProfileResourceSerializer.createForAnnotatedField(userManagerUrlProvider, ANNOTATION);
	}

	@Nested
	class TestSerialize {

		private static final UserId USER_ID = UserId.from(UUID.randomUUID());
		private static final UserId USER_ID_2 = UserId.from(UUID.randomUUID());

		@Mock
		private JsonGenerator generator;
		@Mock
		private SerializerProvider serializerProvider;
		@Captor
		private ArgumentCaptor<Object> writtenObjectCaptor;

		@Nested
		class OnUrlIsConfigured {

			private static final String HTTP_LOCALHOST = "http://localhost/";
			private static final String API_TEMPLATE = "api/profile/%s";
			private static final String API_PATH = "api/profile/";

			@BeforeEach
			void init() {
				when(userManagerUrlProvider.isConfiguredForUserProfile()).thenReturn(true);
				when(userManagerUrlProvider.getUserProfileTemplate()).thenReturn(HTTP_LOCALHOST + API_TEMPLATE);
			}

			@Nested
			class OnSingularValue {

				@SneakyThrows
				@Test
				void shouldSerializeToUrl() {
					serializeSingleValue();

					verify(generator).writeObject(writtenObjectCaptor.capture());
					assertThat(writtenObjectCaptor.getValue()).isEqualTo(expectedUrl(USER_ID));
				}
			}

			@Nested
			class OnCollection {

				@SneakyThrows
				@Test
				void shouldWriteStartArray() {
					serializeCollection();

					verify(generator).writeStartArray();
				}

				@SneakyThrows
				@Test
				void shouldSerializeToUrls() {
					serializeCollection();

					verify(generator, times(2)).writeObject(writtenObjectCaptor.capture());
					assertThat(writtenObjectCaptor.getAllValues()).containsExactly(expectedUrl(USER_ID), expectedUrl(USER_ID_2));
				}

				@SneakyThrows
				@Test
				void shouldWriteEndArray() {
					serializeCollection();

					verify(generator).writeEndArray();
				}
			}

			private String expectedUrl(UserId id) {
				return HTTP_LOCALHOST + API_PATH + id.toString();
			}
		}

		@Nested
		class OnUriIsNotConfigured {

			@BeforeEach
			void init() {
				when(userManagerUrlProvider.isConfiguredForUserProfile()).thenReturn(false);
			}

			@Nested
			class OnSingularValue {

				@SneakyThrows
				@Test
				void shouldSerializeToString() {
					serializeSingleValue();

					verify(generator).writeObject(writtenObjectCaptor.capture());
					assertThat(writtenObjectCaptor.getValue()).isEqualTo(USER_ID.toString());
				}
			}

			@Nested
			class OnCollection {

				@SneakyThrows
				@Test
				void shouldWriteStartArray() {
					serializeCollection();

					verify(generator).writeStartArray();
				}

				@SneakyThrows
				@Test
				void shouldSerializeToUrls() {
					serializeCollection();

					verify(generator, times(2)).writeObject(writtenObjectCaptor.capture());
					assertThat(writtenObjectCaptor.getAllValues()).containsExactly(USER_ID.toString(), USER_ID_2.toString());
				}

				@SneakyThrows
				@Test
				void shouldWriteEndArray() {
					serializeCollection();

					verify(generator).writeEndArray();
				}
			}
		}

		@Nested
		class OnCreatedImplicitlyByObjectMapper {

			@SneakyThrows
			@Test
			void shouldSerializeToString() {
				var objectMapper = new ObjectMapper();

				var serialized = objectMapper.writeValueAsString(new LinkedUserProfileResourceTestObject(UserProfileTestFactory.ID));

				assertThat(serialized).isEqualTo("{\"id\":\"" + UserProfileTestFactory.ID.toString() + "\"}");
			}
		}

		@SneakyThrows
		private void serializeSingleValue() {
			serializer.serialize(USER_ID, generator, serializerProvider);
		}

		@SneakyThrows
		private void serializeCollection() {
			serializer.serialize(List.of(USER_ID, USER_ID_2), generator, serializerProvider);
		}
	}

	@SneakyThrows
	private static LinkedUserProfileResource getAnnotation() {
		var idField = LinkedUserProfileResourceTestObject.class.getDeclaredField("id");
		return idField.getDeclaredAnnotation(LinkedUserProfileResource.class);
	}
}
