/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.forwarding;

import static de.ozgcloud.alfa.common.command.CommandController.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.collaboration.OrganisationsEinheit;
import de.ozgcloud.alfa.collaboration.OrganisationsEinheitController;
import de.ozgcloud.alfa.collaboration.OrganisationsEinheitTestFactory;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandService;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.command.CreateCommand;
import de.ozgcloud.alfa.common.errorhandling.ExceptionController;
import de.ozgcloud.alfa.common.errorhandling.ProblemDetailMapper;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import lombok.SneakyThrows;

class ForwardByVorgangCommandControllerTest {

	@Mock
	private CommandService commandService;
	@Mock
	private OrganisationsEinheitController organisationsEinheitController;
	@Mock
	private ForwardVorgangCommandBodyMapper bodyMapper;
	@Mock
	private ProblemDetailMapper problemDetailMapper;
	@Spy
	@InjectMocks
	private ForwardByVorgangCommandController controller;

	private MockMvc mockMvc;

	@BeforeEach
	void init() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).setControllerAdvice(new ExceptionController(problemDetailMapper)).build();
	}

	@Nested
	class TestCreateCommand {

		private final OrganisationsEinheit organisationsEinheit = OrganisationsEinheitTestFactory.create();
		@Captor
		private ArgumentCaptor<CreateCommand> commandCaptor;
		@Captor
		private ArgumentCaptor<ForwardVorgangCommandBody> commandBodyCaptor;

		@BeforeEach
		void init() {
			when(organisationsEinheitController.getOrganisationEinheit(any())).thenReturn(organisationsEinheit);
			when(commandService.createCommand(any(), anyLong())).thenReturn(CommandTestFactory.create());
			when(bodyMapper.updateFromOrganisationEinheit(any(), any())).thenReturn(ForwardVorgangCommandBodyTestFactory.create());
		}

		@Test
		void shouldCallOrganisationsEinheitController() {
			doRequest();

			verify(organisationsEinheitController).getOrganisationEinheit(OrganisationsEinheitTestFactory.ID);
		}

		@Test
		void shouldAddOrganisationsEinheitData() {
			doRequest();

			verify(bodyMapper).updateFromOrganisationEinheit(same(organisationsEinheit), commandBodyCaptor.capture());
			assertThat(commandBodyCaptor.getValue()).usingRecursiveComparison()
					.isEqualTo(ForwardVorgangCommandBody.builder().organisationEinheitId(OrganisationsEinheitTestFactory.ID).build());
		}

		@Test
		void shouldCallCommandService() {
			doRequest();

			verify(commandService).createCommand(commandCaptor.capture(), eq(VorgangHeaderTestFactory.VERSION));
			assertThat(commandCaptor.getValue()).usingRecursiveComparison().isEqualTo(CreateForwardVorgangCommandTestFactory.create());
		}

		@SneakyThrows
		@Test
		void shouldReturnCreated() {
			var response = doRequest();

			response.andExpect(status().isCreated());
		}

		@SneakyThrows
		@Test
		void shouldReturnLinkToCreatedCommand() {
			var response = doRequest();

			response.andExpect(header().stringValues("location", "http://localhost" + COMMANDS_PATH + "/" + CommandTestFactory.ID));
		}

		@SneakyThrows
		private ResultActions doRequest() {
			return mockMvc.perform(post(ForwardByVorgangCommandController.PATH, VorgangHeaderTestFactory.ID, VorgangHeaderTestFactory.VERSION)
					.contentType(MediaType.APPLICATION_JSON)
					.content(createContent()));
		}

		private String createContent() {
			return CommandTestFactory.buildCreateCommandWithBodyContent(
					CommandOrder.FORWARD_VORGANG,
					"{\"organisationEinheitId\":\"%s\"}".formatted(OrganisationsEinheitTestFactory.ID));
		}
	}
}
